/* LaTeX2Sdoc
 *  Copyright (C) 2000-2001 SAKURAI, Masashi 
 * (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package jp.gr.java_conf.ccs2.tool.tex2sdoc;
import jp.gr.java_conf.ccs2.util.StringUtil;

/**
   TeX macro object
 */
public class Macro {
    String name;
    int argnumber = 0;
    String content;

    /**
       @param name macro name with back slash, ex."\cfrac"
       @param argnumber number of arguments
       @param content macro content
     */
    public Macro(String name,int argnumber,String content) {
	this.name = name;
	this.argnumber = argnumber;
	this.content = content;
    }

    /**
       @param name macro name with back slash, ex."\cfrac"
       @param content macro content
    */
    public Macro(String name,String content) {
	this(name,0,content);
    }

    /**
       @return macro name with back slash
    */
    public String getName() {return name;}

    /**
       @return source of macro content 
    */
    public String getContent() {
	return content;
    }
    public int getArgNumber() {return argnumber;}

    /**
       @reutrn expanded macro content
     */
    public String process(String [] arguments) {
	if (arguments == null || arguments.length == 0) {
	    if (argnumber == 0) return getContent();
	    throw new RuntimeException("Null argument.");
	}
	String ret = getContent();
	for (int i=0;i<argnumber;i++) {
	    ret = StringUtil.replace(ret,"#"+(i+1),arguments[i]);
	}
	return ret;
    }
}
