/*
 *  BibTeX2DOM 
 *  Copyright (C) 2001 SAKURAI, Masashi (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package jp.gr.java_conf.ccs2.tool.tex2sdoc.bibtex;
import jp.gr.java_conf.ccs2.util.ResourceFile;
import jp.gr.java_conf.ccs2.io.EasyWriter;
import jp.gr.java_conf.jaba2.xml.UXML;
import java.util.*;
import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;

public class BibTeX2SBib {

    public static void main(String [] args) {
	String file = "";
	if (args.length == 0) {
	    System.out.println("java BibTeX2SBib (bibtex file)");
	    return;
	} else {
	    file = args[0];
	}
	Document document = getDOMTreeFromFile(file);
	printDocument(document);
    }

    // file -> string -> DOM tree
    public static Document getDOMTreeFromFile(String filename) {
	return getDOMTree( ResourceFile.file2String(filename) );
    }

    public static String getXMLStringFromFile(String filename) {
	return document2String(getDOMTreeFromFile(filename));
    }

    public static Document getDOMTree(String bibtexSource) {
	try {
	    BibTeX2DOM bib = new BibTeX2DOM( bibtexSource );
	    return bib.getDOMTree(newDocument());
	} catch (Exception e) {
	    throw new RuntimeException(e.getMessage());
	}
    }

    public static void translateFile(String input,String output) {
	translateFile(input,output,"Shift_JIS");
    }

    public static void translateFile(String input,String output,
				     String outEncoding) {
	Document doc = getDOMTreeFromFile(input);
	try {
	    EasyWriter writer = new EasyWriter(output,outEncoding);
	    writer.println(UXML.doc2String4Print(doc,outEncoding));
	    writer.close();
	} catch (Exception e) {
	    throw new RuntimeException(e.getMessage());
	}
    }

    //Temporary Document root
    protected static Document newDocument() {
	try {
	    DocumentBuilderFactory factory
		= DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    return builder.newDocument();
	} catch (ParserConfigurationException e) {
	    throw (new InternalError(e.getMessage()));
	}
    }

    protected static String document2String(Document document) {
	try {
	    return UXML.node2String4Print(document.getDocumentElement());
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return null;
    }

    protected static void printDocument(Document document) {
	try {
	    System.out.println(UXML.doc2String4Print(document));
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
