/* LaTeX2Sdoc
 *  Copyright (C) 2000-2001 SAKURAI, Masashi (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package jp.gr.java_conf.ccs2.tool.tex2sdoc.math;
import java.util.*;
import java.lang.reflect.*;

/**
   TeX -> MathML Environment Table
 */
public class MathMLEnvTable {

    static Hashtable table;
    
    public static MathMLEnvHandler getHandler(EnvironmentContent env) {
	if (resources == null && !loadFailed)
	    loadResources();
	try {
	    if (table == null) {
		table = new Hashtable();
	    }

	    String envName = env.getName();

	    Constructor cnst = (Constructor)table.get(envName);
	    if (cnst == null) {
		String shortName = resources.getString(envName);
		if (shortName == null) {
		    System.err.println("no environment :"+envName);
		    return null;
		}
		String cname = shortName;
		Class stc = Class.forName(cname);
		Class [] argsClass = {"".getClass()};
		cnst = stc.getConstructor(argsClass);
		table.put(envName,cnst);
	    }
	    Object [] args = {env.getContent()};
	    MathMLEnvHandler handler = 
		(MathMLEnvHandler)cnst.newInstance(args);
	    return handler;
	} catch (NoSuchMethodException e) {
	    System.err.println("Error! no constructor of the handle-class :"+env.getName());
	} catch (SecurityException e) {
	    System.err.println("Security Exception !");
	    e.printStackTrace();
	} catch (MissingResourceException e) {
	    System.err.println("Error! no environment : "+env.getName());
	    e.printStackTrace();
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return null;
    }

    protected static ResourceBundle resources;
    private static boolean loadFailed = false;

    static void loadResources() {
	try {
	    resources = ResourceBundle.getBundle(
		"jp.gr.java_conf.ccs2.tool.tex2sdoc.math.MathMLEnvironment");
	} catch (MissingResourceException mre) {
	    System.err.println("[MathMLEnvironment.properties] not found");
	    loadFailed = true;
	}
    }
}
