/* LaTeX2Sdoc
 * Copyright (C) 2000-2001 SAKURAI, Masashi (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package jp.gr.java_conf.ccs2.tool.tex2sdoc.math;
import java.util.*;
import jp.gr.java_conf.ccs2.util.StringUtil;

/**
   Tag util
 */
public class TagUtil {

    public static String transEntity(OutputState state, String in) {
	if (state.isEntity()) return in;
	return MathMLTransTable.transEntity(in);
    }

    /** make empty tag with given prepared attribute values */
    public static String makeEmptyTag(OutputState state,
				      String tag,String atts) {
	if (StringUtil.isNull(atts))
	    return tab(state)+"<"+getNS(state)+tag+" /"+getLT(state);
	return tab(state)+"<"+getNS(state)+tag+" "+atts+" /"+getLT(state);
    }

    /** make begin tag with given prepared attribute values */
    public static String makeBtagWithAttrs(OutputState state,
					   String tag,String atts) {
	if (StringUtil.isNull(atts))
	    return tab(state)+"<"+getNS(state)+tag+getLT(state);
	return tab(state)+"<"+getNS(state)+tag+" "+atts+" "+getLT(state);
    }

    /** make begin tag with given attribute value */
    public static String makeBtagWithAttr(OutputState state,String tag,
				  String atn,String atv) {
	return makeBtagWithAttrs(state,tag,atn+"=\""+atv+"\"");
    }

    /** make begin tag with URI attribute */
    public static String makeBtagWithURI(OutputState state,String tag,String uri) {
	if (StringUtil.isNull(state.getNameSpace()))
	    return tab(state)+"<"+tag+" xmlns=\""+state.getXMLURI()+
		"\""+getLT(state);
	return makeBtag(state,tag);
    }

    /** make begin tag */
    public static String makeBtag(OutputState state,String tag) {
	return makeBtagWithAttrs(state,tag,null);
    }

    /** make end tag and CR */
    public static String makeEtag(OutputState state,String tag) {
	return tab(state)+"</"+getNS(state)+tag+getLT(state);
    }

    /** make oneline-tagged content and CR */
    public static String makeTag(OutputState state,String tag,
				 String content) {
	StringBuffer sb = new StringBuffer();
	sb.append(tab(state)+"<"+getNS(state)+tag+">");
	sb.append( content );
	sb.append("</"+getNS(state)+tag+getLT(state));
	return sb.toString();
    }

    /** make tabs */
    public static String tab(OutputState state,int t) {
	if (state.isOneline()) return "";
	StringBuffer sb = new StringBuffer();
	for(int i=0;i<t;i++)
	    sb.append("  ");
	return sb.toString();
    }

    /** make tabs */
    public static String tab(OutputState state) {
	return tab(state,state.getCurrentTab()+state.getStartTab());
    }

    protected static String getLS(OutputState state) {
	if (!state.isOneline()) return "\n";
	return "";
    }

    protected static String getLT(OutputState state) {
	if (!state.isOneline()) return ">\n";
	return ">";
    }

    /** make namespace */
    protected static String getNS(OutputState state) {
	if (state.getNameSpace() == null)
	    return "";
	return state.getNameSpace();
    }

}
