/* LaTeX2Sdoc
 *  Copyright (C) 2000-2001 SAKURAI, Masashi (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package jp.gr.java_conf.ccs2.tool.tex2sdoc.math;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import jp.gr.java_conf.ccs2.comp.TreeStructureViewer;
import jp.gr.java_conf.ccs2.core.EasyMain;
import jp.gr.java_conf.ccs2.util.TreeStructure;
import jp.gr.java_conf.ccs2.tool.tex2sdoc.TeXUtil;

public class Test extends EasyMain {
    
    public static void main(String [] args) {
	boolean tree = false;
	TreeStructureViewer viewer = null;
	while(true) {
	    String test = read("Input tex source ([.]:quit)","");

	    if (test == null || test.equals(""))
		continue;
	    if (test.equals(".")) break;
	    if (test.equals("tree")) {
		tree = true;
	    }
	    if (test.equals("test")) {
		selfTest();
		break;
	    }

	    TreeStructure treest = translate( test );
	    if (tree) {
		if (viewer == null)
		    viewer = new TreeStructureViewer( treest );
		else
		    viewer.setTree( treest );
		continue;
	    }
	}
    }

    static TreeStructure translate(String input) {
	Expression exp = null;
	try {
	    TeX2MathML eng = new TeX2MathML();
	    eng.setEntity(true);
	    exp = eng.getObjectTree( TeXUtil.prepareTeXSource(input) );
	    print( eng.translate( input ));
	    print( eng.getLog());
	    print("<br/>");
	} catch (TeX2MathMLException e) {
	    print(e.getMessage());
      	} catch (ParseException e) {
	    print(e.getMessage());
      	} catch (TokenMgrError e) {
	    print(e.getMessage()+
		  "\n-----\n"+input+"\n----\n  row:"+
		  TeX2MathML.getErrorRow(e)+"  col:"+
		  TeX2MathML.getErrorCol(e));
	}
	return exp;
    }

    static void selfTest() {
	String [] tests = {
	    "x^2+5x+6=0","\\frac{1}{(1+n^2)}","\\sum_j^{\text{{eXg}}",
	    "x=0 \\quad\\text{then,}\\quad f(x)=0",
	    "\\text{The vector} \\quad\\vec{v} = \\binom{a}{b}",
	    "\\overleftarrow{AB} \\quad\\stackrel{\\text{def}}{\\longleftarrow} \\quad\\overbrace{AB}^\\text{all}",
	    "\\cfrac{-1}{1+\\cfrac{1}{(1-n^2)}}",
	    "\\sqrt{\\frac{\\pi}{a}} = {\\displaystyle\\int_{-\\infty}^\\infty e^{-ax^2}\\>\\mathrm{d}x}",
	    "\\left(\\begin{array}{cc}1&-\\gamma\\\\-\\gamma&1\\end{array}\\right)",
	    "\\begin{array}{rcl}\\Xi(\\mu, T, V)  &=&  \\sum_{N=0}^{\\infty}\\frac{h^{-3N}}{N!}\\exp[N\\beta \\mu] \\int\\!\\!\\!\\int \\exp[-\\beta \\mathcal{H}_N(\\mathbf{r}^{N}, \\mathbf{p}^N)]  \\mbox{$\\mathrm{d}  \\mathbf{r}  $} ^{N}  \\mbox{$\\mathrm{d}  \\mathbf{p}  $} ^{N} \\nonumber \\\\ &=&  \\sum_{N=0}^{\\infty}\\frac{z^N}{N!}Z_N(V, T)\\end{array}",
	    "\\begin{split}x_1 = 1 & x_2 = 2& x_3 = 0\\\\y_1 = 1 & y_2 = 2& y_3 = 0 \\end{split}",
	    "\\begin{multiline} \\cfrac{\\partial \\langle N \\rangle}{\\partial \\beta\\mu}  \\\\ =    \\cfrac{\\partial^2 \\log\\Xi}{\\partial (\\beta\\mu)^2} \\nonumber\\\\  =  \\cfrac{ \\cfrac{\\partial^2 \\log\\Xi}{\\partial (\\beta\\mu)^2}\\Xi -   (\\cfrac{\\partial \\Xi}{\\partial \\beta\\mu})^2 }{\\Xi^2} \\nonumber \\\\ =  \\langle N^2 \\rangle -\\langle N \\rangle^2 \\end{multiline}",
	    "|x| = \\begin{cases}x & when $(x > 0)$ \\\\ 0 & $x = 0$\\\\-x&$x<0$ \\end{cases}"};
	for (int i=0;i<tests.length;i++) {
	    translate( tests[i] );
	}
    }

}
