/* LaTeX2Sdoc
 * Copyright (C) 2000-2001 SAKURAI, Masashi (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package jp.gr.java_conf.ccs2.tool.tex2sdoc.math.handler;
import jp.gr.java_conf.ccs2.tool.tex2sdoc.math.*;
import jp.gr.java_conf.ccs2.util.StringUtil;

/**
   MathML root Command handler implements class
 */
public class RootHandler implements MathMLCommandHandler {
    
    private static final String MROOT= "mroot";
    private static final String MSQRT= "msqrt";
    private static final String MROW= "mrow";

    CommandContent content;

    public RootHandler() { }
    public RootHandler(CommandContent c) {content = c; }

    public boolean hasOption() {return true;}
    public int getArgNumber() {return 1;}
    public boolean isTextMode(){ return false;}

    public void onCommandBegin(MathMLVisitor parent) {
	if (StringUtil.isNull(content.getOption()))
	    parent.addString(TagUtil.makeBtag(parent,MSQRT));
	else
	    parent.addString(TagUtil.makeBtag(parent,MROOT));
    }

    public void onCommandEnd(MathMLVisitor parent) {
	if (StringUtil.isNull(content.getOption()))
	    parent.addString(TagUtil.makeEtag(parent,MSQRT));
	else
	    parent.addString( TagUtil.makeEtag(parent,MROOT) );
    }

    public void onOption(MathMLVisitor parent) {}

    public void onArgumentBegin(MathMLVisitor parent) {
	parent.addString( TagUtil.makeBtag(parent,MROW) );
    }

    public void onArgumentEnd(MathMLVisitor parent) {
	parent.addString( TagUtil.makeEtag(parent,MROW) );
	if (!StringUtil.isNull(content.getOption()))
	    parent.addString( TagUtil.makeTag(parent,MROW,content.getOption()) );
    }
    
    public String toString() {
	String ret = "Cmd["+MROOT+"] ";
	return ret;
    }

    public MathMLCommandHandler getCopy(CommandContent c) {
	return new RootHandler(c);
    }

}
