/* LaTeX2Sdoc
 *  Copyright (C) 2000-2001 SAKURAI, Masashi (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package jp.gr.java_conf.ccs2.tool.tex2sdoc.math.handler;
import jp.gr.java_conf.ccs2.tool.tex2sdoc.math.*;
import jp.gr.java_conf.ccs2.util.StringUtil;

/**
   MathML simple Command handler implements class
 */
public class SimpleCommandHandler implements MathMLCommandHandler {

    protected String cmdTag;
    protected String [] argTags;
    protected String [] argAttrs;
    protected String preChar,postChar;//pre and post string
    protected String attribute;
    protected int [] indexies;//argument index

    /**
       @param tcmdTag tag and attribute (Ex.  mrow@displaystyle=\"\true" )
       @param targTags array of argument tag
     */
    public SimpleCommandHandler(String tcmdTag,
				String [] targTags) {
	this(tcmdTag,targTags,null,null,null);
    }
    public SimpleCommandHandler(
	String tcmdTag,String [] targTags,
	int [] indexies,String preChar,String postChar) {

	this.indexies = indexies;
	this.preChar = preChar;
	this.postChar = postChar;

	String [] rets = StringUtil.split(tcmdTag,"@");
	this.cmdTag = rets[0];
	if (rets.length == 2)
	    this.attribute = rets[1];

	if (targTags != null && targTags.length != 0) {
	    this.argTags = new String[targTags.length];
	    this.argAttrs = new String[targTags.length];
	    for (int i=0;i<argTags.length;i++) {
		rets = StringUtil.split(targTags[i],"@");
		this.argTags[i] = rets[0];
		if (rets.length == 2)
		    this.argAttrs[i] = rets[1];
	    }
	}
    }

    //for copy
    protected SimpleCommandHandler() {
    }


    public boolean hasOption() {return false;}
    public void onOption(MathMLVisitor parent) {}
    public int getArgNumber() {
	if (argTags == null) return 0;
	return argTags.length;
    }
    public boolean isTextMode(){ return false;}

    int argIndex = 0;
    public void onCommandBegin(MathMLVisitor parent) {
	if ( !StringUtil.isNull(preChar) ) {
	    parent.addString( TagUtil.tab(parent)+preChar );
	}

	if ( StringUtil.isNull(cmdTag) ) return;

	if ( getArgNumber()==0 ) {
	    parent.addString( TagUtil.makeEmptyTag(
		parent,cmdTag,attribute));
	    return;
	}

	parent.addString(
	    TagUtil.makeBtagWithAttrs(parent,cmdTag,attribute));
	argIndex = 0;
    }

    public void onCommandEnd(MathMLVisitor parent) {
	if ( getArgNumber()==0 ||
	     StringUtil.isNull(cmdTag) ) return;
	parent.addString( TagUtil.makeEtag(parent,cmdTag) );

	if ( !StringUtil.isNull(postChar) ) {
	    parent.addString( TagUtil.tab(parent)+postChar );
	}
    }

    public void onArgumentBegin(MathMLVisitor parent) {
	String tt = argTags[argIndex];
	if ( StringUtil.isNull(tt) ) return;
	parent.addString( 
	    TagUtil.makeBtagWithAttrs(
		parent,tt,argAttrs[argIndex]) );
    }
    
    public void onArgumentEnd(MathMLVisitor parent) {
	String tt = argTags[argIndex];
	if ( StringUtil.isNull(tt) ) return;
	parent.addString( TagUtil.makeEtag(parent,tt) );
	argIndex++;
    }
    
    public String toString() {
	String ret = "MathML ["+cmdTag+"] ";
	if (!StringUtil.isNull(attribute)) ret += "["+attribute+"] ";
	if (argTags == null || argTags.length == 0) return ret;
	for(int i=0;i<argTags.length;i++) {
	    ret += "\n";
	    ret += "ARG:"+argTags[i]+" ";
	    if (!StringUtil.isNull(argAttrs[i]))
		ret += "["+argAttrs[i]+"] ";
	}
	if (indexies != null) {
	    String a = "";
	    for(int i=0;i<indexies.length;i++)
		a += indexies[i];
	    ret += "\nARRANGE{"+a+"}";
	}
	if (!StringUtil.isNull(preChar)) {
	    ret += "\nPRE:"+preChar;
	}
	if (!StringUtil.isNull(postChar)) {
	    ret += "\nPST:"+postChar;
	}
	return ret;
    }

    protected void copyFrom(SimpleCommandHandler p) {
	this.cmdTag = p.cmdTag;
	this.attribute = p.attribute;
	this.argTags = p.argTags;
	this.argAttrs = p.argAttrs;
	this.indexies = p.indexies;
    }
    
    public MathMLCommandHandler getCopy(CommandContent c) {
	SimpleCommandHandler s = new SimpleCommandHandler();
	s.copyFrom(this);
	arrangeArgsIndexies(c);
	return s;
    }

    protected void arrangeArgsIndexies(CommandContent c) {
	if (indexies == null) return;
	c.arrangeArguments(indexies);
    }
}
