/* CCS, Class Collection by Sakurai
 *  Copyright (C) 2000-2001 SAKURAI, Masashi (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package jp.gr.java_conf.ccs2.util;

import java.sql.*;
import java.util.ArrayList;
import jp.gr.java_conf.ccs2.util.StringUtil;


public class JDBCUtil {

    public static boolean registDriver(String driverClassName) {
	try {
	    Class.forName(driverClassName).newInstance(); 
	    return true;
	}
	catch (Exception e) {
	    System.err.println("Unable to load driver ["+driverClassName+"].");
	    e.printStackTrace();
	}
	return false;
    }

    public static ResultSet getResultSet(Connection con,String sql) throws SQLException {
	Statement stmt = null;
        try {
	    stmt = con.createStatement();
	    ResultSet set = stmt.executeQuery(sql);
            return set;
        } finally {
	    if (stmt != null)
		stmt.close();
	}
    }

    public static ResultSet getResultSet(String url,String sql) throws SQLException {
	Connection con = null;
        try {
	    con = DriverManager.getConnection(url);
	    return getResultSet(con,sql);
        } finally {
	    if (con != null)
		con.close();
	}
    }
    
    public static String CTRL = "%";

    public static String [] layoutData(String layout,ResultSet rs) throws SQLException {
	if (StringUtil.isNull(layout)) return null;
	if (rs == null) return clearLayoutDatas(layout);
	ResultSetMetaData rsmd = rs.getMetaData();
	int cols = rsmd.getColumnCount();
	String [] columnNames = null;
	int [] columns = null;
	if (layout.indexOf(CTRL+"#") != -1) {
	    //contain index name
	    columnNames = new String[cols*2];
	    columns = new int[cols*2];
	    for (int i=0;i<cols;i++) {
		columnNames[i] = CTRL+rsmd.getColumnName(i+1)+CTRL;
		columnNames[i+cols] = CTRL+"#"+(i+1)+CTRL;
		columns[i] = i+1;
		columns[i+cols] = i+1;
	    }
	} else {
	    columnNames = new String[cols];
	    columns = new int[cols];
	    for (int i=0;i<cols;i++) {
		columnNames[i] = CTRL+rsmd.getColumnName(i+1)+CTRL;
		columns[i] = i+1;
	    }
	}
	rs.last();
	int cc = rs.getRow();
	String [] rets = new String[cc];
	rs.beforeFirst();
	cc = 0;
	while (rs.next()) {
	    String line = layout;
	    for (int i=0;i<columnNames.length;i++) {
		String buf = rs.getString(columns[i]);
		line = StringUtil.replace(line,columnNames[i],buf);
	    }
	    rets[cc] = line;
	    cc++;
	}
	return rets;
    }

    public static String [] clearLayoutDatas(String in) {
	String [] ret = {clearLayoutData(in)};
	return ret;
    }

    public static String clearLayoutData(String in) {
	StringBuffer ret = new StringBuffer();
	while (true) {
	    int ps = in.indexOf(CTRL);
	    if (ps == -1) break;
	    ret.append(in.substring(0,ps));
	    in = in.substring(ps+1);
	    int pe = in.indexOf(CTRL);
	    if (pe == -1) break;
	    in = in.substring(pe+1);
	}
	ret.append(in);
	return ret.toString();
    }
}
